pro phasecal_cross, caldiffxy, caldiffyx, phasechnls, dpdf, frq, $
                    ozero, oslope, phase_observed, $
                    sigma_phasefit=sigma_phasefit, rf=rf, tfit=tfit

;+
; NAME:
;       PHASECAL_CROSS
;
; PURPOSE: 
;       Least squares fit for the phase of the correlated cal versus
;       frequency, and return the phase offset OZERO and slope OSLOPE. The
;       phase versus frequency is fit to the equation...  phase = ozero +
;       oslope*frq
;
; CALLING SEQUENCE:
;       PHASECAL_CROSS, caldiffxy, caldiffyx, phasechnls, dpdf, frq, $
;                    ozero, oslope, phase_observed, $
;                    sigma_phasefit=sigma_phasefit, rf=rf, tfit=tfit
;
; INPUTS: 
;       CALDIFFXY[ nchnls] - the XY [CALON - CALOFF] correlated outputs
;
;       CALDIFFYX[ nchnls] - the YX [CALON - CALOFF] correlated outputs
;
;       PHASECHNLS: the particular chnls to include in the fit (from
;                   structure SCNDATA).
;
;       DPDF: the initial guess for (dphase/dfreq). UNITS ARE RADIANS/MHZ. 
;
;       FRQ[ nchnls]: the aray of frequencies, centered at the frequency at
;                     which you want the zero of phase to be
;                     calculated. UNITS ARE MHZ. If RF is set, WHICH IT
;                     SHOULD BE, then FRQ is the RF freq. If RF is not
;                     set, FRQ is baseband freq. These days, with
;                     wideband receivers and multiple sub-bands being
;                     sampled simultaneously, you should SET RF=1!!
;
; KEYWORDS: 
;       SIGMA_PHASEFIT, the sigma of the phasegradient fit
;       RF. See note below under MOD HIISTORY. If set, 
;                phase = ozero + oslope* frq  (frq is RF frq, MHz)
;       If RF is NOT set,
;                phase = ozero + oslope* (frq - mean(frq[phasechnls]))
;       TFIT - the fitted datapoints
; OUTPUTS:
;       OZERO[2] - the zero intercept and its 1sigma error, in RADIANS
;
;       OSLOPE[2] - the phase slope and its 1sigma error, in RADIANS/MHz
;
;       PHASE_OBSERVED - the array of observed (not the fitted) phases of
;                        the cal deflection, UNITS ARE RADIANS
;
; MODIFICATION HISTORY:
;       3 feb 2019: added keyword RF so that if RF is set, phase is given by 
;         phase = ozero + oslope* frq    [where frq is RF freq MHz]. 
;       This was done for FAST. All software previous to this used RF=0,
;       i.e. RF NOT set. 
;               BEST TO SET RF!!!
;-
;stop
;phasegradient_fit, frq[phasechnls], $
;                   caldiffxy[phasechnls], caldiffyx[phasechnls], dpdf, $
;                   coeffs, sigcoeffs, aphi, sigaphi, sigma_phasefit, tfit
phasegradient_fit, double(frq[phasechnls]), $
    double(caldiffxy[phasechnls]), double(caldiffyx[phasechnls]), $
    double(dpdf), coeffs, sigcoeffs, aphi, sigaphi, sigma_phasefit, tfit

;stop

; CONVERT COEFFS AND APHI TO OZERO AND OSLOPE IN 
;    PHASE = OZERO + OSLOPE*(FREQ-FREQZERO)
; IN THE 2-ELEMENT ARRAYS, FIRST ONE IS VALUE AND SECOND IS ERROR.
; ***** THE UNITS ARE RADIANS ********
ozero=dblarr(2)
oslope=dblarr(2)
ozero[0] = aphi
if keyword_set(rf) then ozero[0]= aphi - coeffs[2]*mean(frq[phasechnls])
ozero[1] = sigaphi
;STOP
oslope[0] = coeffs[2]
oslope[1] = sigcoeffs[2]

phase_observed = atan(caldiffyx, caldiffxy)

end ; phasecal_cross


