pro products_to_stokes_2021, gainchnls, cal_indx, cts_per_k_xx, cts_per_k_yy, $
                phase_zero, phase_slope, freq, $
                xx1, xx0, yy1, yy0, $
                xy1, xy0, yx1, yx0, $
                stki_meas, stkq_meas, stku_meas, stkv_meas, $
                xxcalib, yycalib, xycalib, yxcalib, $
                no_intensitycal=no_intensitycal, mean_med_off=mean_med_off

;+
;PURPOSE: turn calibrated self- and cross-products into the 'measured'
;       Stokes parameters.
;
;CALLING SEQUENCE:
;PRODUCTS_TO_STOKES, gainchnls, cal_indx, cts_per_k_xx, cts_per_k_yy, $
;                phase_zero, phase_slope, freq, $
;                xx1, xx0, yy1, yy0, $
;                xy1, xy0, yx1, yx0, $
;                stki_meas, stkq_meas, stku_meas, stkv_meas, $
;                xxcalib, yycalib, xycalib, yxcalib, $
;                no_intensitycal=no_intensitycal, mean_med_off=mean_med_off
;
;INPUTS:
;GAINCHNLS[nrchnls]: A vector of spectral channel numbers that is being
;       used to determine power gains
;CAL_INDX[nrscns]: A vector that specifies which CAL goes with which
;       SRC. For example, if SRC has 98 scans and CAL has 9 scans, CAL_INDX is
;       a 98-element vector with values ranging from 0 to 8.
;CTS_PER_K_XX, CTS_PER_K_YY: the counts per kelvin that tells
;       the power gain for X and Y signal paths.
;PHASE_ZERO[2]: a 2-element vector telling the phase and its error
;       for the zeroth baseband frequency. Units are RADIANS.
;PHASE_SLOPE[2]: a 2-element vector telling the phase slope and its
;       error. Units are RADIANS per MHz.
;FREQ[nrchnls]: the array of freqs of the gainchnls, MHz
;XX1[nrchnls, nrscns], XX0[nrchnls, nrscns]: xx product onsource, $
;       xx product offsource.  
;YY1, YY0: same as for XX1 products immediately above.
;XY1, XY0: same as for XX1 products above.
;YX1, YX0: same as for XX1 products above.
;
;KEYWORDS:
;NO_INTENSITYCAL: Set this keyword if you wish to NOT do
;       intensity calibration. In some Arecibo reductions, the intensity
;       calibration has been done previously, in which case this keyword should
;       be set. Otherwise, don't set it.
;MEAN_MED_OFF: if nonzero, use the mean or median of the
;       off-source product spectra (instead of paired onsource/offsource
;       spectra) to calculate calibrated switched spectra. MEAN_MED_OFF=1 uses
;       the mean and MEAN_MED_OFF=2 uses the median.
;                                                               
;OUTPUTS
;STKI_MEAS[ nrchnls, nrscans], etc: the 'measured' Stokes parameters calculated from
;XXCALIB[ nrchnls, nrscans], etc: (the calibrated xx, etc. voltage product spectra.
;-

sz= size( xx1)
nchnls= sz[ 1]
nrscans= sz[ 2]

xxcalib= fltarr( nchnls, nrscans)
yycalib= fltarr( nchnls, nrscans)
xxbandpass= fltarr( nchnls, nrscans)
yybandpass= fltarr( nchnls, nrscans)
xycalib_int= fltarr( nchnls, nrscans)
yxcalib_int= fltarr( nchnls, nrscans)
xycalib= fltarr( nchnls, nrscans)
yxcalib= fltarr( nchnls, nrscans)

;BANDPASS CORRECT:
for nscn= 0l, nrscans-1l do begin
   self_uncal_to_cal, xx1[ *, nscn], xx0[ *, nscn], gainchnls, $
                 cts_per_k_xx[cal_indx[ nscn]], xxcal, xxbp, $
                 xxtsyson, xxtsysoff, $
                 no_intensitycal=no_intensitycal, mean_med_off=mean_med_off, $
                 /pswitch
xxcalib[ *, nscn]= xxcal
xxbandpass[ *, nscn]= xxbp
   self_uncal_to_cal, yy1[ *, nscn], yy0[ *, nscn], gainchnls, $
                 cts_per_k_yy[cal_indx[ nscn]], yycal, yybp, $
                 yytsyson, yytsysoff, $
                 no_intensitycal=no_intensitycal, mean_med_off=mean_med_off, $
                 /pswitch
yycalib[ *, nscn]= yycal
yybandpass[ *, nscn]= yybp
endfor

;INTENSITY CALIBRATE
for nscn= 0l, nrscans-1l do begin
cross_uncal_to_cal, xy1[ *, nscn], gainchnls, $
                    cts_per_k_xx[cal_indx[ nscn]], xxbandpass[ *,nscn], $
                    cts_per_k_yy[cal_indx[ nscn]], yybandpass[ *,nscn], $
                    xycal, $
                    /pswitch, uncal_cross_ref=xy0, $
                    no_intensitycal=no_intensitycal
xycalib_int[ *, nscn]= xycal

cross_uncal_to_cal, yx1[ *, nscn], gainchnls, $
                    cts_per_k_xx[cal_indx[ nscn]], xxbandpass[ *,nscn], $
                    cts_per_k_yy[cal_indx[ nscn]], yybandpass[ *,nscn], $
                    yxcal, $
                    /pswitch, uncal_cross_ref=yx0, $
                    no_intensitycal=no_intensitycal
yxcalib_int[ *, nscn]= yxcal

;stop

;PHASE CORRECT
phasecorr_xyyx, xycalib_int[*, nscn], yxcalib_int[*, nscn], freq, $
    phase_zero[ cal_indx[ nscn], 0], phase_slope[ cal_indx[ nscn], 0], $
    xy, yx
xycalib[ *, nscn]= xy
yxcalib[ *, nscn]= yx
endfor

;finally...
stki_meas= xxcalib+ yycalib
stkq_meas= xxcalib- yycalib
stku_meas= 2.* xycalib
stkv_meas= 2.* yxcalib

return
end
