pro stereographic, long, lat, x, y, southpole=southpole, $
        inverse=inverse, gnomic=gnomic, equi=equi, radius=radius, lefthand=lefthand

;+
; NAME: STEREOGRAPHIC PURPOSE: Convert long and lat to X,Y, or vice-versa,
; using a stereographic projection. Also does a Gnomic or an azimuthal
; equidistant projection.
;
; EXPLANATION:
;       The output X and Y coordinates are normalized so that latitude 0
;       (the equator) lies at radius R=1 [R = sqrt(x^2 + y^2)] The pole has
;            R=0 and lies at lat=90.
;       Output points can be centered on the north pole or south pole.
;
; CALLING SEQUENCE:
;       STEREOGRAPHIC, long, lat, X, Y, $
;        [ /SOUTHPOLE ], [ /lefthand ], [ /INVERSE ], [ /GNOMIC], [ /EQUI]
;
; INPUTS:
;       long - longitude - scalar or vector, in degrees
;       lat - latitude - same number of elements as long, in degrees
;
; OUTPUTS:
;       X - X coordinate, same number of elements as long.   X is normalized
;           so that 90 deg away from the pole occurs at radius = 1
;       Y - Y coordinate, same number of elements as long.  Y is normalized
;           like X
;
; KEYWORDS:
;       /SOUTHPOLE - Keyword to indicate that the plot is to be centered on
;                    the south pole instead of the north pole.
;       /INVERSE - go from X,Y to long, lat
;       /GNOMIC - do Gnomic instead of Stereographic projection
;       /EQUI - do Azimuthal Equidistant instead of Stereographic projection
;       /LEFTHAND - make the projection image left-handed with respect
;                   to the long/lat system (which means: increasing
;                   longitude goes clockwise instead of
;                   counterclockwise) -
;
; HISTORY:
;09jan2018: CH added 'equi' option, cleaned up software and made
;/south behave correctly (I think!)  -
  
  latmult=1
  xmult=1.0
  ymult=1.0
;if keyword_set( lefthand) then xmult=-1. else xmult=1.
if keyword_set( lefthand) then ymult=-ymult
if keyword_set( southpole) then begin
   ymult=-ymult
   latmult=-latmult
endif

if ( keyword_set( gnomic) or keyword_set( equi) ) then begin
   two=1. 
   half=1.
endif else begin
   two=2.
   half=0.5
endelse

if keyword_set(inverse) then begin
   radius= sqrt( x^2 + y^2)
   if keyword_set(equi) eq 0 then $
;      lat= -90.+ two*!radeg*atan(radius) else $
;      lat= -90.+ radius*90.
      lat= latmult*(90.- two*!radeg*atan(radius)) else $
      lat= latmult*(90.- radius*90.)
   long=!radeg*atan( y*ymult, x*xmult)  
endif

   if keyword_set( equi) eq 0 then $
      radius = tan( !dtor* half* (90.- latmult*lat)) else $
      radius = (90.- latmult*lat)/90.
   x= xmult* radius* cos(!dtor* long)
   y= ymult* radius* sin(!dtor* long)

;stop
return
end

