pro psopen, filename, $
            LANDSCAPE=landscape, $
            XSIZE=xsize, $
            YSIZE=ysize, $
            INCHES=inches, $
            COLOR=color, $
            ENCAPSULATED=encapsulated, $
            BITS_PER_PIXEL=bits_per_pixel, $
            _REF_EXTRA=_extra
;+
; NAME:
;       PSOPEN
;     
; PURPOSE:
;       To open the PostScript device for outputting graphics to a file.
;     
; CALLING SEQUENCE:
;       PSOPEN, FILENAME [, /LANDSCAPE] [, XSIZE=width] [, YSIZE=height] [,
;       /INCHES] [, /COLOR] [, BITS_PER_PIXEL={1 | 2 | 4 | 8}] [, 
;       /ENCAPSULATED]
;
; Other DEVICE keywords accepted:
;       [, /AVANTGARDE | , /BKMAN | , /COURIER | , /HELVETICA | ,
;          /ISOLATIN1 | , /PALATINO | , /SCHOOLBOOK | , /SYMBOL | , 
;          /TIMES | , /ZAPFCHANCERY | , /ZAPFDINGBATS ] [, 
;        /BOLD] [, /BOOK] [, /DEMI] [, FONT_INDEX=integer] [, 
;       FONT_SIZE=points] [,  GLYPH_CACHE=number_of_glyphs] [,
;       /ITALIC] [, /LIGHT] [, /MEDIUM] [, /NARROW] [, /OBLIQUE] [, 
;       OUTPUT=scalar string] [, SCALE_FACTOR=value] [, 
;       SET_CHARACTER_SIZE=[font size, line spacing]] [, 
;       SET_FONT=scalar string] [, /TT_FONT] [, 
;       XOFFSET=value] [, YOFFSET=value]
;
; INPUTS:
;       FILENAME : String with the name of the PostScript file to be opened.
;     
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       /LANDSCAPE: If set, landscape orientation is used. Portrait 
;                   orientation is the default.
;
;       XSIZE = The width of output generated by IDL. XSIZE is specified 
;               in centimeters, unless /INCHES is set.
;
;       YSIZE = The height of output generated by IDL. YSIZE is specified 
;               in centimeters, unless /INCHES is set.
;       
;       XOFFSET = The X position, on the page, of the lower left corner of 
;                 output generated by IDL. XOFFSET is specified in 
;                 centimeters, unless /INCHES is set.
;
;       YOFFSET = The Y position, on the page, of the lower left corner of 
;                 output generated by IDL. YOFFSET is specified in 
;                 centimeters, unless /INCHES is set.
;
;       /INCHES : Normally, the XOFFSET, XSIZE, YOFFSET, and YSIZE keywords 
;                 are specified in centimeters. However, if INCHES is 
;                 present and non-zero, they are taken to be in inches 
;                 instead.
;
;       /COLOR: Set this keyword to enable color PostScript output.
;
;       BITS_PER_PIXEL = The number of bits per pixel to use. IDL is capable 
;                        of producing PostScript images with 1, 2, 4, or 8 
;                        bits per pixel. Using more bits per pixel gives 
;                        higher resolution at the cost of generating larger 
;                        files.  The default value is 8 bits per pixel.
;
;       /ENCAPSULATED: Set this keyword to create an encapsulated 
;                      PostScript file, suitable for importing into another 
;                      document (e.g., LaTeX). The file extension will be 
;                      ".eps" rather than the default ".ps".
;
;       See "Keywords Accepted by the IDL Devices" in the IDL Online Help.  
;       Only keywords followed by {PS} are applicable to the PostScript 
;       device.
;
; COMMON BLOCKS:
;       None.
;
; SIDE EFFECTS:
;	The graphics device is set to PostScript.
;       If FILENAME is sent in without a ".ps" or ".eps" suffix, it is
;       returned with one appended to it.  Also, if the requested
;       filename has a ".ps" suffix and the /ENCAPSULATED keyword is
;       set, the file will be saved with a ".eps" suffix.  It is
;       standard operating procedure for encapsulated PostScript
;       files to end with ".eps" and this is enforced by ApJ, so
;       we make it so.
;
; RESTRICTIONS:
;       PSOPEN assumes letter-sized pages.
;       PSOPEN sets the graphics device to PostScript and leaves a
;       file open upon completion.  Therefore, after all graphics
;       have been created for this file, it is necessary to close
;       the file (DEVICE, /CLOSE_FILE) and set the graphics device
;       back to the default for your machine (probably X Windows).
;       PSCLOSE will do all of this for you.
;
; EXAMPLE:
;       Make a PostScript file 5in by 5in:
;         IDL> psopen, 'foo.ps', XSIZE=5, YSIZE=5, /INCHES
;         IDL> plot, findgen(30)^2
;         IDL> psclose
;
;       Make a color PostScript image:
;         IDL> psopen, 'foo.ps', /COLOR
;         IDL> setcolors, /SYSTEM_VARIABLES
;         IDL> plot, findgen(30), color=!red
;         IDL> oplot, findgen(30)^2, color=!blue
;         IDL> xyouts, 0.25, 0.5, /normal, 'Parabola', color=!green
;         IDL> xyouts, 0.55, 0.5, /normal, 'Line', color=!magenta
;         IDL> psclose
;         IDL> setcolors, /SYSTEM_VARIABLES
;
;       Make a PostScript image taking advantage of PostScript fonts
;       AND the ability (unique in IDL fontdom) of PS fonts to have
;       embedded formatting command indices changed:
;         IDL> psopen, 'foo.ps', /HELVETICA, /BOLD, /OBLIQU, /ISOLATIN1
;         IDL> device, /BKMAN, /DEMI, /ITALIC, /ISOLATIN1, FONT_INDEX=10
;         IDL> plot, findgen(3), FONT=0, $
;         IDL>   xtit='Galactic Radius !10'+string(174B)+'!X [kpc]', $
;         IDL>   ytit='Density !10'+string(181B)+'!X [cm!E-3!N]'
;         IDL> psclose
;
;       To make a CMYK image for publication in a journal, just
;       set the /CMYK keyword in your call to PSOPEN.  This requires IDL
;       v6.1 or above.
;
; NOTES:
;       Add device keywords if you want to change font characteristics
;       but remember that in order to use PostScript fonts you must either 
;       set the !P.FONT system variable to 0 (so that IDL uses the 
;       hardware fonts) or send any of the plotting routines the 
;       FONT keyword set to 0.
;       
;       Remember that the PostScript device is 8-bit and has exactly
;       256 color table indices.  So, on your X-windows device, if
;       you're running 8-bit PseudoColor, you've probably got less
;       than 256 color table indices available... if you're running
;       24-bit TrueColor or DirectColor, you've got 16 million color
;       indices and no color table at all.  If you're storing color
;       table indices in variables, you'll need to reassign these
;       variables with the correct color table indices after you've
;       opened the PostScript device (and again when you've closed
;       it!)
;
;       I moved to Australia where A4 is the standard paper size.  I wanted
;       to allow PSOPEN to size images for A4 in a backwards-compatible
;       way.  Therefore, PSOPEN will default to LETTER size unless the
;       system variable ${PAPERSIZE} is set to a paper size (e.g., A4).
;       That said, I'm only checking for the subset of ISO and
;       North-American non-ISO paper sizes that are listed in the ghostview
;       program.  If you want something not listed, you'll have to
;       edit this program.
;
; RELATED PROCEDURES:
;       PSCLOSE
;
; MODIFICATION HISTORY:
;       Written by Tim Robishaw in ancient times.
;       27 Feb 2002  Spiffed up by TR
;       28 Jan 2004  Cosmetic changes. TR
;	19 Jul 2006  Added example of the new (v6.1) /CMYK device keyword.
;       01 Dec 2008  Added ${PAPERSIZE} stuff. TR.
;-

; LOOK FOR A POSTSCRIPT EXTENSION...
filename = strtrim(filename,2)
dotpos = strpos(filename,'.',/reverse_search)
if (dotpos ge 0) then begin

    ; STRIP THE CURRENT SUFFIX SO WE CAN APPEND PROPER SUFFIX...
    suffix = strmid(filename,dotpos)
    if strcmp(suffix,'.ps',/FOLD_CASE) OR strcmp(suffix,'.eps',/FOLD_CASE) $
      then filename = strmid(filename,0,dotpos)

endif

; APPEND THE PROPER EXTENSION...
filename = filename + (keyword_set(ENCAPSULATED) ? '.eps' : '.ps')

; DEFAULT BITS_PER_PIXEL IS 8...
if (N_elements(BITS_PER_PIXEL) eq 0) then bits_per_pixel = 8

; CHECK TO SEE IF THE USER HAS SET THE ${PAPERSIZE} ENVIRONMENT VARIABLE...
papersize = getenv('PAPERSIZE')

; SET THE PAPER SIZE IN UNITS OF CENTIMETERS...
case strlowcase(papersize) of
   ; ISO SIZES...
   'a3' : papersz = {x:29.7,y:42.0}
   'a4' : papersz = {x:21.0,y:29.7}
   'a5' : papersz = {x:14.8,y:21.0}
   'a6' : papersz = {x:10.5,y:14.8}
   'b4' : papersz = {x:25.0,y:35.3}
   'b5' : papersz = {x:17.6,y:25.0}
   'b6' : papersz = {x:12.5,y:17.6}
   ; NON-ISO NORTH-AMERICAN SIZES...
   'letter'  : papersz = {x:21.6,y:27.9}
   'legal'   : papersz = {x:21.6,y:35.6}
   'ledger'  : papersz = {x:27.9,y:43.2}
   'tabloid' : papersz = {x:27.9,y:43.2}
   'quarto'  : papersz = {x:22.9,y:27.9}
   'statement' : papersz = {x:14.0,y:21.6}
   'folio'   : papersz = {x:21.0,y:33.0}
   ; IF ${PAPERSIZE} IS NOT SET, WE ASSUME LETTER PAPERSIZE...
   '' : papersz = {x:21.6,y:27.9}
   else : begin
      message, /INFO, '${PAPERSIZE} '+strtrim(papersize,2)+' unknown;'+$
     ' PSOPEN.PRO needs updating. Assuming letter size for now.'
      papersz = {x:21.6,y:27.9}
   end
endcase

; DEFAULT IS THAT SIZES ARE IN CENTIMETERS, SO IF XSIZE AND YSIZE ARE
; PASSED IN IN INCHES, THEN /INCHES KEYWORD MUST BE SET...
if keyword_set(INCHES) then begin
   papersz.x = papersz.x/2.54
   papersz.y = papersz.y/2.54
endif

; IF XSIZE, YSIZE, XOFFSET OR YOFFSET ARE SENT IN VIA _REF_EXTRA, 
; THOSE VALUES WILL OVERRIDE THE ONES BELOW...
if keyword_set(LANDSCAPE) then begin

    ; SET UP LANDSCAPE ORIENTATION...
    if (N_elements(XSIZE) eq 0) OR (N_elements(YSIZE) eq 0) then begin
        xsize = 0.95*papersz.y
        ysize = 0.95*papersz.x
    endif

    ; KEEP THE IMAGE CENTERED ON PAGE...
    xoffset = 0.5*(papersz.x - ysize)
    yoffset = 0.5*(papersz.y - xsize) + xsize

endif else begin

    ; SET UP PORTRAIT ORIENTATION...
    if (N_elements(XSIZE) eq 0) OR (N_elements(YSIZE) eq 0) then begin
        xsize = 0.95*papersz.x
        ysize = 0.95*papersz.y
    endif

    ; KEEP THE IMAGE CENTERED ON PAGE...
    xoffset = 0.5*(papersz.x - xsize)
    yoffset = 0.5*(papersz.y - ysize)

endelse

; SET THE OUTPUT DEVICE FOR IDL GRAPHICS TO POSTSCRIPT...
set_plot, 'PS'

; SET THE DEVICE PARAMETERS...
device, FILENAME  = filename, $
        LANDSCAPE = keyword_set(LANDSCAPE), $
        XSIZE     = xsize, $
        YSIZE     = ysize, $
        XOFFSET   = xoffset, $
        YOFFSET   = yoffset, $
        INCHES    = keyword_set(INCHES), $
        COLOR     = keyword_set(COLOR), $
        BITS_PER_PIXEL = bits_per_pixel, $
        ENCAPSULATED   = keyword_set(ENCAPSULATED), $
        _EXTRA    = _extra

end; psopen
