
pro posang_conv, long1, lat1, posang1, $
                 long2, lat2, posang2, $
                 rtot_12=rtot_12, inverse=inverse, $
                 eq_to_gal=eq_to_gal, equinox=equinox

;+
;Convert the (long,lat,pa) of linear pol in coordsys 1 to those in
;       coordsys 2 using rotation matrix rtot_12
;POSANG_CONV, long1, lat1, posang1, $
;                 long2, lat2, posang2, $
;                 rtot_12=rtot_12, inverse=inverse, $
;                 eq_to_gal=eq_to_gal, equinox=equinox
;
;INPUTS/OUTPUTS:
;if (LONG1,LAT1,POSANG1) are given and you want (LONG2,LAT2,POSANG2)
;       then set INVERSE=0
;if (LONG2,LAT2,POSANG2) are given and you want (LONG1,LAT1,POSANG1)
;       then set INVERSE=1
;LONG1, LAT1, POSANG1: the long,lat,posang in system 1. units are DEGREES
;LONG2, LAT2, POSANG2: the long,lat,posang in system 2. units are DEGREES
;
;KEYWORDS:
;RTOT_12: the rotation matrix that converts system 1 to system 2. you
;       can use
;       RMATRIX, longpole, latpole, rtot_12, [longoffset=longoffset]
;INVERSE: long2, etc are inputs, long1, etc are outputs
;       e.g., equatorial to galactic:
;       POSANG_CONV, ra, dec, pa_eq, gl, gb, pa_gal,/eqtogal
;       e.g., galactic to equatorial:
;       POSANG_CONV, ra, dec, pa_eq, gl, gb, pa_gal, /eqtogal, /inverse
;EQ_TO_GAL: system 1 is equatorial, system 2 is Galactic. RTOT_12 is
;       defined and you can use the returned value
;EQUINOX: for EQ_TO_GAL, specify EQUINOX (2000 is default).
;
;EXAMPLE 1:
;       you have a position in Galactic
;       coords (ell, bee, pa_gal) and you want the (long2,lat2,pa_2)
;       in the new system whose pole lies at Galactic coords
;       (longpole, latpole).
;       FIRST, get the rotation matrix RTOT_12:
;     RMATRIX, longpole, latpole, rtot_12, [longoffset=longoffset]
;       THEN apply POSANG_CONV
;       THEN 
;       POSANG_CONV, ell, bee, pa_gal, rtot_12, long2, lat2, pa2
;OR: you have the position (long2,lat2,pa_2) and you want
;       the quantities in Galactic coords:
;       POSANG_CONV, ell, bee, pa_gal, rtot_12, long2, lat2, pa2, /INVERSE
;
;HISTORY:
;checked and documented by CH 09dec2017
;-

if keyword_set( eq_to_gal) then begin
if n_elements( equinox) eq 0 then equinox=2000.d0
glactc, longpole, latpole, equinox, 0.d0, 90.d0, 2, /deg
;longpole= 192.85948
;latpole= 27.128302
longoffset= 57.068
rmatrix, longpole, latpole, rtot_12, longoffset=longoffset, /noleft
endif

;stop, '1'

if keyword_set(inverse) then begin
   long_in=long2
   lat_in=lat2
   posang_in=posang2
   rtot_12_inv= transpose(rtot_12)
endif else begin
   long_in=long1
   lat_in=lat1
   posang_in=posang1
   rtot_12_inv= rtot_12
endelse

;stop, 2

;FIRST, GET THE POSITIONS IN SYSTEM_OUT:
sph_coord_conv, long_in, lat_in, rtot_12_inv, long_out, lat_out
long_out= modangle(long_out, 360.)

;print,'From posang_conv:'
;print,long_in
;print, long_out
;print, lat_in
;print, lat_out


;stop,'3'

;NEXT, THE END POINTS OF LINE SEGMENTS 0.2 DEG LONG...
del_long_in= 0.1*sin(!dtor* posang_in)/ cos(!dtor*lat_in) 
del_lat_in= 0.1*cos(!dtor* posang_in)
long_inm= modangle( long_in- del_long_in, /negpos)
lat_inm= modangle( lat_in- del_lat_in, /negpos)
long_inp= modangle( long_in+ del_long_in, /negpos)
lat_inp= modangle( lat_in+ del_lat_in, /negpos)
sph_coord_conv, long_inm, lat_inm, rtot_12_inv, long_outm, lat_outm;, inverse=inverse
sph_coord_conv, long_inp, lat_inp, rtot_12_inv, long_outp, lat_outp;, inverse=inverse
;del_long_out= long_outp- long_outm
;del_lat_out= lat_outp- lat_outm
del_long_out= modangle( long_outp- long_outm, /negpos)
del_lat_out= modangle( lat_outp- lat_outm, 180, /negpos)

posang_out= atan( del_long_out* cos(!dtor* lat_out), del_lat_out)/ !dtor
posang_out= modangle( posang_out, 180.)

if n_elements( long_out) eq 1 then long_out=long_out[0]
if n_elements( lat_out) eq 1 then lat_out=lat_out[0]

if keyword_set(inverse) then begin
   long1= long_out
   lat1= lat_out
   posang1= posang_out
   endif else begin
   long2= long_out
   lat2= lat_out
   posang2= posang_out
endelse

return

end
